import os

BOARD = os.getenv('BOARD')

if BOARD == "raspi_40pin":
    from raspberrypi.raspi_40pin import *

elif BOARD == "raspberry_pi_cm":
    from raspberrypi.raspi_cm import *

elif BOARD == "RASPBERRY_PI_B_REV1":
    from raspberrypi.raspi_1b_rev1 import *

elif BOARD == "RASPBERRY_PI_A" or BOARD == "RASPBERRY_PI_B_REV2":
    from raspberrypi.raspi_1b_rev2 import *
else:
	# by default load RPI B rev 2
	from raspberrypi.raspi_1b_rev2 import *

def I2C():
    """The singleton I2C interface"""
    import busio

    return busio.I2C(SCL, SDA)


def SPI():
    """The singleton SPI interface"""
    import busio

    return busio.SPI(SCLK, MOSI, MISO)


"""
Mock Library for RPi.GPIO
"""

import time
import logging
import os
import yaml

logger = logging.getLogger(__name__)

log_level = os.getenv('LOG_LEVEL')

if log_level is not None:
    if log_level == "Info":
        logger.setLevel(logging.INFO)
    if log_level == "Debug":
        logger.setLevel(logging.DEBUG)
    if log_level == "Warning":
        logger.setLevel(logging.WARNING)
    if log_level == "Error":
        logger.setLevel(logging.ERROR)
    if log_level == "Critical":
        logger.setLevel(logging.CRITICAL)
else:
    logger.setLevel(logging.ERROR)

stream_formatter = logging.Formatter('%(asctime)s:%(levelname)s: %(message)s')
stream_handler = logging.StreamHandler()
stream_handler.setFormatter(stream_formatter)
logger.addHandler(stream_handler)

BCM = 11
BOARD = 10
BOTH = 33
FALLING = 32
HARD_PWM = 43
HIGH = 1
I2C = 42
IN = 1
LOW = 0
OUT = 0
PUD_DOWN = 21
PUD_OFF = 20
PUD_UP = 22
RISING = 31
RPI_INFO = {'MANUFACTURER': 'Sony', 'P1_REVISION': 3, 'PROCESSOR': 'BCM2837', 'RAM': '1G', 'REVISION': 'a020d3', 'TYPE': 'Pi 3 Model B+'}
RPI_REVISION = 3
SERIAL = 40
SPI = 41
UNKNOWN = -1
VERSION = '0.7.0'

_mode = 0

channel_config = {}

#flags
setModeDone = False

class Channel:
    def __init__(self,channel, direction, initial=0,pull_up_down=PUD_OFF):
        self.chanel = channel
        self.direction = direction
        self.initial = initial
        self.pull_up_down = pull_up_down


#GPIO LIBRARY Functions
def setmode(mode):
    """
    Set up numbering mode to use for channels.
    BOARD - Use Raspberry Pi board numbers
    BCM   - Use Broadcom GPIO 00..nn numbers
    """
    # GPIO = GPIO()
    time.sleep(1)
    if(mode == BCM):
        setModeDone = True
        _mode = mode

    elif (mode == BOARD):
        setModeDone = True
    else:
        setModeDone = False

def getmode():
    """
    Get numbering mode used for channel numbers.
    Returns BOARD, BCM or None
    """
    return _mode

def setwarnings(flag):
    """
    Enable or disable warning messages
    """
    logger.info("Set Warings as {}".format(flag))

def setup(channel, direction, initial=0,pull_up_down=PUD_OFF):
    """
    Set up a GPIO channel or list of channels with a direction and (optional) pull/up down control
    channel        - either board pin number or BCM number depending on which mode is set.
    direction      - IN or OUT
    [pull_up_down] - PUD_OFF (default), PUD_UP or PUD_DOWN
    [initial]      - Initial value for an output channel

    """
    logger.info("setup channel : {} as {} with intial :{} and pull_up_dowm {}".format(channel,direction,initial,pull_up_down))
    global channel_config
    channel_config[channel] = Channel(channel, direction, initial, pull_up_down)

def output(channel, value):
    """
    Output to a GPIO channel or list of channels
    channel - either board pin number or BCM number depending on which mode is set.
    value   - 0/1 or False/True or LOW/HIGH

    """
    logger.info("output channel : {} with value : {}".format(channel, value))

def input(channel):
    """
    Input from a GPIO channel.  Returns HIGH=1=True or LOW=0=False
    channel - either board pin number or BCM number depending on which mode is set.
    """
    logger.info("reading from chanel {}".format(channel))

def wait_for_edge(channel,edge,bouncetime,timeout):
    """
    Wait for an edge.  Returns the channel number or None on timeout.
    channel      - either board pin number or BCM number depending on which mode is set.
    edge         - RISING, FALLING or BOTH
    [bouncetime] - time allowed between calls to allow for switchbounce
    [timeout]    - timeout in ms
    """
    logger.info("waiting for edge : {} on channel : {} with bounce time : {} and Timeout :{}".format(edge,channel,bouncetime,timeout))


def add_event_detect(channel,edge,callback,bouncetime):
    """
    Enable edge detection events for a particular GPIO channel.
    channel      - either board pin number or BCM number depending on which mode is set.
    edge         - RISING, FALLING or BOTH
    [callback]   - A callback function for the event (optional)
    [bouncetime] - Switch bounce timeout in ms for callback
    """
    logger.info("Event detect added for edge : {} on channel : {} with bouce time : {} and callback {}".format(edge,channel,bouncetime,callback))

def event_detected(channel):
    """
    Returns True if an edge has occurred on a given GPIO.  You need to enable edge detection using add_event_detect() first.
    channel - either board pin number or BCM number depending on which mode is set.
    """
    logger.info("Waiting for even detection on channel :{}".format(channel))

def add_event_callback(channel,callback):
    """
    Add a callback for an event already defined using add_event_detect()
    channel      - either board pin number or BCM number depending on which mode is set.
    callback     - a callback function
    """
    logger.info("Event Calback : {} added for channel : {}".format(callback,channel))

def remove_event_detect(channel):
    """
    Remove edge detection for a particular GPIO channel
    channel - either board pin number or BCM number depending on which mode is set.
    """
    logger.info("Event Detect Removed for channel : {}".format(channel))

def gpio_function(channel):
    """
    Return the current GPIO function (IN, OUT, PWM, SERIAL, I2C, SPI)
    channel - either board pin number or BCM number depending on which mode is set.
    """
    logger.info("GPIO function of Channel : {} is {}".format(channel,channel_config[channel].direction))


class PWM:
    # initialise PWM channel
    def __init__(self, channel, frequency):
        """
        x.__init__(...) initializes x; see help(type(x)) for signature
        """
        self.chanel = channel
        self.frequency = frequency
        self.dutycycle = 0
        global channel_config
        channel_config[channel] = Channel(channel,PWM,)
        logger.info("Initialized PWM for Channel : {} at frequency : {}".format(channel,frequency))
    
    # where dc is the duty cycle (0.0 <= dc <= 100.0)
    def start(self, dutycycle):
        """
        Start software PWM
        dutycycle - the duty cycle (0.0 to 100.0)
        """
        self.dutycycle = dutycycle
        logger.info("start pwm on channel : {} with Duty cycle : {}".format(self.chanel,dutycycle))
    
    # where freq is the new frequency in Hz
    def ChangeFrequency(self, frequency):
        """
        Change the frequency
        frequency - frequency in Hz (freq > 1.0)
        """
        logger.info("Freqency Changed for channel : {} from : {} -> to : {}".format(self.chanel,self.frequency,frequency))
        self.frequency = frequency

    # where 0.0 <= dc <= 100.0
    def ChangeDutyCycle(self, dutycycle):
        """
        Change the duty cycle
        dutycycle - between 0.0 and 100.0
        """
        self.dutycycle = dutycycle
        logger.info("Dutycycle Changed for channel : {} from : {} -> to : {}".format(self.chanel,self.dutycycle,dutycycle))
    
    # stop PWM generation
    def stop(self):
        logger.info("Stop pwm on channel : {} with Duty cycle : {}".format(self.chanel,self.dutycycle))


def cleanup(channel=None):
    """
    Clean up by resetting all GPIO channels that have been used by this program to INPUT with no pullup/pulldown and no event detection
    [channel] - individual channel or list/tuple of channels to clean up.  Default - clean every channel that has been used.
    """
    if channel is not None:
        logger.info("Cleaning Up Channel : {}".format(channel))
    else:
        logger.info("Cleaning Up all channels")


# # The MIT License (MIT)
# #
# # Copyright (c) 2018 Carter Nelson for Adafruit Industries
# #
# # Permission is hereby granted, free of charge, to any person obtaining a copy
# # of this software and associated documentation files (the "Software"), to deal
# # in the Software without restriction, including without limitation the rights
# # to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# # copies of the Software, and to permit persons to whom the Software is
# # furnished to do so, subject to the following conditions:
# #
# # The above copyright notice and this permission notice shall be included in
# # all copies or substantial portions of the Software.
# #
# # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# # IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# # FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# # AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# # THE SOFTWARE.
# """
# ``Adafruit_TCA9548A``
# ====================================================
#
# CircuitPython driver for the TCA9548A I2C Multiplexer.
#
# * Author(s): Carter Nelson
#
# Implementation Notes
# --------------------
#
# **Hardware:**
#
# * TCA9548A I2C Multiplexer: https://www.adafruit.com/product/2717
#
# **Software and Dependencies:**
#
# * Adafruit CircuitPython firmware for the supported boards:
#   https://github.com/adafruit/circuitpython/releases
# * Adafruit's Bus Device library: https://github.com/adafruit/Adafruit_CircuitPython_BusDevice
# """
#
# # imports
import time
# from micropython import const

_DEFAULT_ADDRESS = 0x70

__version__ = "0.0.0-auto.0"
__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_TCA9548A.git"


class TCA9548A_Channel:
    """Helper class to represent an output channel on the TCA9548A and take care
       of the necessary I2C commands for channel switching. This class needs to
       behave like an I2CDevice."""

    def __init__(self, tca, channel):
        self.tca = tca
        self.channel_switch = bytearray([1 << channel])

    # def try_lock(self):
    #     """Pass thru for try_lock."""
    #     # while not self.tca.i2c.try_lock():
    #     #     pass
    #     # self.tca.i2c.writeto(self.tca.address, self.channel_switch)
    #     time.sleep(1)
    #     return True
    #
    # def unlock(self):
    #     """Pass thru for unlock."""
    #     self.tca.i2c.writeto(self.tca.address, b"\x00")
    #     return self.tca.i2c.unlock()
    #
    # def readfrom_into(self, address, buffer, **kwargs):
    #     """Pass thru for readfrom_into."""
    #     if address == self.tca.address:
    #         raise ValueError("Device address must be different than TCA9548A address.")
    #     return self.tca.i2c.readfrom_into(address, buffer, **kwargs)
    #
    # def writeto(self, address, buffer, **kwargs):
    #     """Pass thru for writeto."""
    #     if address == self.tca.address:
    #         raise ValueError("Device address must be different than TCA9548A address.")
    #     return self.tca.i2c.writeto(address, buffer, **kwargs)
    #
    # def writeto_then_readfrom(self, address, buffer_out, buffer_in, **kwargs):
    #     """Pass thru for writeto_then_readfrom."""
    #     # In linux, at least, this is a special kernel function call
    #     if address == self.tca.address:
    #         raise ValueError("Device address must be different than TCA9548A address.")
    #
    #     if hasattr(self.tca.i2c, "writeto_then_readfrom"):
    #         self.tca.i2c.writeto_then_readfrom(address, buffer_out, buffer_in, **kwargs)
    #     else:
    #         self.tca.i2c.writeto(
    #             address,
    #             buffer_out,
    #             start=kwargs.get("out_start", 0),
    #             end=kwargs.get("out_end", None),
    #             stop=False,
    #         )
    #         self.tca.i2c.readfrom_into(
    #             address,
    #             buffer_in,
    #             start=kwargs.get("in_start", 0),
    #             end=kwargs.get("in_end", None),
    #         )


class TCA9548A:
    """Class which provides interface to TCA9548A I2C multiplexer."""
    def __init__(self, i2c, address=_DEFAULT_ADDRESS):
        self.i2c = i2c
        self.address = address
        self.channels = [None] * 8

    def __len__(self):
        return 8

    def __getitem__(self, key):
        if not 0 <= key <= 7:
            raise IndexError("Channel must be an integer in the range: 0-7")
        if self.channels[key] is None:
            self.channels[key] = TCA9548A_Channel(self, key)
        return self.channels[key]



from setuptools import setup, find_packages

from distutils.util import convert_path

long_description ="""
# Virtual Pi

The easiest way to use this package is to install using pip3 for python 3

```bash
$ sudo pip3 install VPi
```

To use the mock or virtual pi just type the following at the beginning of your script.

```python
try:
      from RPi.GPIO import GPIO
      import board
      import busio
except:
      from VPi.GPIO import GPIO
      import VPi.board as board
      import VPi.busio as busio
```

## Works with

- [python 3.6.8](https://www.python.org/downloads/release/3.6.8)

"""

pkg_ns = {}

ver_path = convert_path('VPi/__init__.py')
with open(ver_path) as ver_file:
    exec(ver_file.read(), pkg_ns)

setup(
      name='VPi',
      version=pkg_ns['__version__'],
      description='Virtual Pi Library for Raspberry Pi',
      url='https://github.com/codenio/',
      author='Aananth K',
      author_email='aananthraj1995@gmail.com',
      license='GPL-3.0',
      packages=find_packages(exclude=[]),
      install_requires=[],
      zip_safe=False,
      long_description_content_type="text/markdown",
      long_description=long_description,
)